# This file contains functions to create a new day for Advent of Code
import logging
import os
import shutil
from jinja2 import Environment, PackageLoader

#Jinja2 environment
env = Environment(
    loader=PackageLoader('pyaoc', '.'),
)

# Logger
logger = logging.basicConfig(level=logging.WARNING, format='%(levelname)s %(message)s')

def create_day(day_number: int = 1) -> int:
    """Create a new directory for a new day of Advent of Code.

    Returns an int that represents if an error occured:
        * 0: No error
        * 1: Error
    
    :param int day_number: Number of the day. Must be between 1 and 25.

    :return: error
    :rtype: int
    """
    # Check if the day number is valid
    if day_number < 1 or day_number > 25:
        logging.error("Invalid day number : CREATE_DAY must be between 1 and 25")
        return 1
    
    # Create the directory
    dir_name = f"day{day_number:02d}"


    # Create folders
    if os.path.exists(dir_name):
        logging.error(f"Directory {dir_name} already exists")
        return 1
    
    os.mkdir(dir_name)
    os.chdir(dir_name)

    # Input folder
    if os.path.exists("inputs"):
        logging.error("inputs folder already exists")
        return 1
    
    os.mkdir("inputs")

    # Benchmark folder
    if os.path.exists("benchmark"):
        logging.error("benchmark folder already exists")
        return 1
    
    os.mkdir("benchmark")

    # Create files

    template = env.get_template("part.jinja2")


    # input and sample files
    os.chdir("inputs")
    with open("input.txt", "w") as f:
        pass
    with open("sample.txt", "w") as f:
        pass
    os.chdir("..")

    # part1 file
    with open("part1.py", "w") as f:
        f.write(template.render(day_number=day_number, part_number="1"))

    # part2 file
    with open("part2.py", "w") as f:
        f.write(template.render(day_number=day_number, part_number="2"))

    # return to the root directory
    os.chdir("..")

    return 0



def create_day_force(day_number: int = 1) -> int:
    """Create a new directory for a new day of Advent of Code. If the directory already exists, delete it and create a new one.
    
    Returns an int that represents if an error occured:
        * 0: No error
        * 1: Error
    
    :param int day_number: Number of the day. Must be between 1 and 25.

    :return: error
    :rtype: int
    """
    # Check if the day number is valid
    if day_number < 1 or day_number > 25:
        logging.error("Invalid day number : CREATE_DAY must be between 1 and 25")
        return 1
    
    # Create the directory
    dir_name = f"day{day_number:02d}"

    # Check if the directory already exists
    if os.path.exists(dir_name):
        # Delete the directory
        shutil.rmtree(dir_name)
    
    # Create folders

    os.mkdir(dir_name)
    os.chdir(dir_name)

    # Input folder
    if os.path.exists("inputs"):
        shutil.rmtree("inputs")
    os.mkdir("inputs")

    # Benchmark folder
    if os.path.exists("benchmark"):
        shutil.rmtree("benchmark")
    os.mkdir("benchmark")


    # Create files

    template = env.get_template("part.jinja2")

    # input and sample files
    os.chdir("inputs")
    with open("input.txt", "w") as f:
        pass
    with open("sample.txt", "w") as f:
        pass
    os.chdir("..")

    # part1 file
    with open("part1.py", "w") as f:
        f.write(template.render(day_number=day_number, part_number="1"))

    # part2 file
    with open("part2.py", "w") as f:
        f.write(template.render(day_number=day_number, part_number="2"))

    # return to the root directory
    os.chdir("..")

    return 0 # No error


# This file contains functions to run a day of Advent of Code
import logging
import os
import time
from importlib.util import spec_from_file_location, module_from_spec

# Logger
logger = logging.basicConfig(level=logging.WARNING, format='%(levelname)s %(message)s')


def run_specific_day(day_number: int = 1, timed: bool = False) -> int:
    """Run a specific day of Advent of Code.

    Returns an int that represents if an error occured:
        * 0: No error
        * 1: Error

    :param int day_number: Number of the day. Corresponding directory must exist.
    :param bool timed: The run is timed.
    
    :return: error
    :rtype: int
    """
    if day_number < 1 or day_number > 25:
        logging.error("Invalid day number : RUN_DAY must be between 1 and 25")
        return 1

    dir_name = f"day{day_number:02d}"
    
    if not os.path.exists(dir_name):
        logging.error(f"Directory {dir_name} does not exist")
        return 1
    
    os.chdir(dir_name)

    text_input = open("inputs/input.txt", "r").read()

    # Part 1
    spec = spec_from_file_location("part1", "part1.py")
    part1 = module_from_spec(spec)
    spec.loader.exec_module(part1)

    if timed:
        start_p1 = time.time()
    res_p1 = part1.solution(text_input)
    if timed:
        end_p1 = time.time()
    print(f"Part 1: {res_p1}")
    if timed:
        print(f"Part 1 execution time: {end_p1 - start_p1:.5f}s")

    # Part 2
    spec = spec_from_file_location("part2", "part2.py")
    part2 = module_from_spec(spec)
    spec.loader.exec_module(part2)

    if timed:
        start_p2 = time.time()
    res_p2 = part2.solution(text_input)
    if timed:
        end_p2 = time.time()
    print(f"Part 2: {res_p2}")
    if timed:
        print(f"Part 2 execution time: {end_p2 - start_p2:.5f}s")

    os.chdir("..")
    
    return 0

def run_specific_part_specific_day(part_number: int = 1, day_number: int = 1, timed: bool = False) -> int:
    """Run a specific part of a specific day of Advent of Code.

    Returns an int that represents if an error occured:
        * 0: No error
        * 1: Error

    :param int part_number: Number of the part. Corresponding file must exist.
    :param int day_number: Number of the day. Corresponding directory must exist.

    :return: error
    :rtype: int
    """
    if part_number < 1 or part_number > 2:
        logging.error("Invalid part number : PART must be between 1 and 2")
        return 1

    if day_number < 1 or day_number > 25:
        logging.error("Invalid day number : DAY must be between 1 and 25")
        return 1

    dir_name = f"day{day_number:02d}"
    
    if not os.path.exists(dir_name):
        logging.error(f"Directory {dir_name} does not exist")
        return 1
    
    os.chdir(dir_name)

    text_input = open("inputs/input.txt", "r").read()

    spec = spec_from_file_location(f"part{part_number}", f"part{part_number}.py")
    part = module_from_spec(spec)
    spec.loader.exec_module(part)

    if timed:
        start = time.time()
    res = part.solution(text_input)
    if timed:
        end = time.time()
    print(f"Part {part_number}: {res}")
    if timed:
        print(f"Part {part_number} execution time: {end - start:.5f}s")
    
    os.chdir("..")

    return 0


def run_current(timed: bool = False) -> int:
    """Run the current day of Advent of Code.

    Returns an int that represents if an error occured:
        * 0: No error
        * 1: Error

    :param bool timed: The run is timed.
        
    :return: error
    :rtype: int
    """
    # Get the current day
    current_path = os.getcwd()
    current_day = os.path.basename(current_path)
    day_number = current_day[3:]
    if current_day[:3] != 'day' and not day_number.isdigit():
        logging.error("Invalid current day folder")
        return 1
    
    day_number = int(day_number)
    os.chdir("..")
    err = run_specific_day(day_number, timed)
    os.chdir(current_path)
    return err

def run_current_specific_part(part_number: int = 1, timed: bool = False) -> int:
    """Run a specific part of the current day of Advent of Code.

    Returns an int that represents if an error occured:
        * 0: No error
        * 1: Error

    :param int part_number: Number of the part. Corresponding file must exist.
    :param bool timed: The run is timed.

    :return: error
    :rtype: int
    """
    # Get the current day
    current_path = os.getcwd()
    current_day = os.path.basename(current_path)
    day_number = current_day[3:]
    if current_day[:3] != 'day' and not day_number.isdigit():
        logging.error("Invalid current day folder")
        return 1
    
    day_number = int(day_number)

    os.chdir("..")
    err = run_specific_part_specific_day(part_number, day_number, timed)
    os.chdir(current_path)
    return err


# This file contains functions to get information about a day of Advent of Code
import requests
import re

def get_day_name(day_number: int = 1, year_number: int = 2023) -> str:
    """Get the name of a specific day of a specific year of the Advent Of Code

    :param int day_number: Number of the day. Must be between 1 and 25
    :param int year_number: Number of the year. Must be between 2015 and current year

    :return: day name
    :rtype: str
    """
    url = f"https://adventofcode.com/{year_number}/day/{day_number}"

    req = requests.get(url)
    full_title = re.findall("<h2>---.*Day \d+:.*---</h2>", req.text)[0] # To get the full day title
    title = re.findall(":\s.*\s", full_title)[0][2:-1]

    return title


import argparse
import logging
import sys
from pyaoc.day_benchmark import benchmark_specific_day
from pyaoc.day_creation import create_day, create_day_force
from pyaoc.day_launch import run_specific_day, run_specific_part_specific_day, run_current, run_current_specific_part
from pyaoc.day_year import get_last_aoc_year
from pyaoc.readme_generation import generate_readme

VERSION = "0.0.1"

logger = logging.basicConfig(level=logging.WARNING, format='%(levelname)s %(message)s')

def main():
    # Create the argument parser
    parser = argparse.ArgumentParser()

    # Add the arguments
    parser.add_argument('--create-day', '-d', type=int, metavar="DAY", help = 'Create a new day')
    parser.add_argument('--force', '-f', action='store_true', help = 'Force the creation of a new day')
    parser.add_argument('--run', '-r', type=int, metavar="DAY", help='Run a specific day')
    parser.add_argument('--run-current', '-rc', action='store_true', help='Run the current day folder')
    parser.add_argument('--part', '-p', type=int, metavar="PART", help='Run a specific part of a specific day')
    parser.add_argument('--time', '-t', action='store_true', help='Print the execution time of each part')
    parser.add_argument('--benchmark', '-b', type=int, nargs=2, metavar=("DAY", "ITERATIONS"), help='Run a specific day with a specific number of iterations')
    parser.add_argument('--year', '-y', type=int, metavar="YEAR", help="Override the current year for the command")
    parser.add_argument('--readme', action="store_true", help="Generate the README file")
    parser.add_argument('--version', action='version', version=f'pyaoc {VERSION}')
    
    # Parse the arguments
    args = parser.parse_args()

    if len(sys.argv) == 1: # No arguments
        parser.print_help()
        return 0
    

    if args.force and not args.create_day:
        logging.error("Force option can only be used with create-day option")
        return 1
    
    if args.part and not (args.run or args.run_current):
        logging.error("Part option can only be used with run option")
        return 1
    
    if args.time and not (args.run or args.run_current):
        logging.error("Time option can only be used with run option or run-current option")
        return 1
    
    if args.run and args.run_current:
        logging.error("Run option and run-current option are mutually exclusive")
        return 1
    
    if args.create_day:
        if args.force:
            err = create_day_force(args.create_day)
        else:
            err = create_day(args.create_day)

        if err:
            return 1
        else:
            print(f"Day {args.create_day} created")
    
    if args.run:
        if args.part:
            err = run_specific_part_specific_day(args.part, args.run, args.time)
        else:
            err = run_specific_day(args.run, args.time)
            
        if err:
            return 1
        
    if args.run_current:
        if args.part:
            err = run_current_specific_part(args.part, args.time)
        else:
            err = run_current(args.time)
            
        if err:
            return 1 

    if args.benchmark:
        err = benchmark_specific_day(args.benchmark[0], args.benchmark[1])
        if err:
            return 1
        
    year = get_last_aoc_year()
    if args.year:
        year = args.year

    if args.readme:
        err = generate_readme()
        if err:
            return 1
        else:
            print("README generated")
    return 0



# This file contains functions to get the year of Advent of Code

import time

def get_last_aoc_year() -> int:
    """Compute the last Advent Of Code year.

    Return the last Advent Of Code year. If you are in decembre it return the current year.
    Otherwise, it return the last year.

    :return: year
    :rtype: int
    """
    now = time.time()
    est = time.gmtime(now - 5 * 60 * 60) # EST is equal to UTC-5

    current_year = est.tm_year
    current_month = est.tm_mon

    if current_month == 12:
        return current_year
    else:
        return current_year - 1


# This file contains functions to benchmark a day of Advent of Code
import logging
import os
import platform
import timeit
from importlib.util import spec_from_file_location, module_from_spec


# Logger
logger = logging.basicConfig(level=logging.WARNING, format='%(levelname)s %(message)s')

def benchmark_specific_day(day_number: int = 1, iterations: int = 1000) -> int:
    """Benchmark a specific day of Advent of Code.

    Return an int that represents if an error occured:
        * 0: No error
        * 1: Error

    :param int day_number: Number of the day. Corresponding directory must exist.
    :param int iterations: Number of iterations.

    :return: error
    :rtype: int      
    """
    if day_number < 1 or day_number > 25:
        logging.error("Invalid day number : RUN_DAY must be between 1 and 25")
        return 1

    dir_name = f"day{day_number:02d}"

    if not os.path.exists(dir_name):
        logging.error(f"Directory {dir_name} does not exist")
        return 1
    
    os.chdir(dir_name)

    text_input = open("inputs/input.txt", "r").read()

    spec = spec_from_file_location("part1", "part1.py")
    part1 = module_from_spec(spec)
    spec.loader.exec_module(part1)

    time_p1 = timeit.timeit(lambda: part1.solution(text_input), number=iterations) / iterations
    
    spec = spec_from_file_location("part2", "part2.py")
    part2 = module_from_spec(spec)
    spec.loader.exec_module(part2)

    time_p2 = timeit.timeit(lambda: part2.solution(text_input), number=iterations) / iterations


    with open("benchmark/benchmark.txt", "w") as f:
        f.write(f"System: {platform.system()}\n")
        f.write(f"Processor: {platform.processor()}\n")
        f.write(f"Bit architecture: {platform.architecture()[0]}\n")
        f.write(f"Python version: {platform.python_version()}\n")
        f.write(f"Day: {day_number}\n")
        f.write(f"Part 1: {time_p1:.5f}s mean time for {iterations} iterations\n")
        f.write(f"Part 2: {time_p2:.5f}s mean time for {iterations} iterations\n")

    print(open("benchmark/benchmark.txt", "r").read())

    os.chdir("..")

    return 0


# This file contains functions to generate the README.md
import os
from jinja2 import Environment, PackageLoader
from pyaoc.day_subject import get_day_name

#Jinja2 environment
env = Environment(
    loader=PackageLoader('pyaoc', '.'),
)

def correct_name_folder(name: str) -> bool:
    """Check that the folder have a name in following format : day{day_number}

    Where {day_number} is a number between 01 and 25.

    :param int name: Name of the folder.

    :return: valide
    :rtype: bool
    """
    if len(name) != 5:
        return False
    
    day_number = name[3:]
    if name[:3] != 'day' and not day_number.isdigit():
        return False
    
    day_number = int(day_number)

    if day_number < 1 or day_number > 25:
        return False
    
    return True


def generate_readme(year: int = 2023) -> int:
    """Generate the README

    Returns an int that represents if an error occured:
        * 0: No error
        * 1: Error

    :param int part_number: Number of the part. Corresponding file must exist.
    :param int year_number: Number of the year. Must be between 2015 and current year

    :return: error
    :rtype: int
    """
    days = []
    files = os.listdir()
    for file in files:
        if os.path.isdir(file) and correct_name_folder(file):
            days.append(file)
    
    data = []
    for day in days:
        number = int(day[3:])
        name = get_day_name(number, year)
        benchmark = open(os.path.join(day, "benchmark", "benchmark.txt"), "r").read()
        if len(benchmark) == 0:
            benchmark = "Not benchmarked\n"
        data.append((day[3:], name, benchmark))

    template = env.get_template("README.jinja2")

    data.sort()

    with open("README.md", "w") as f:
        f.write(template.render(days = data, year=year))

    return 0





